require "active_support/hash_with_indifferent_access"
require "active_support/core_ext/hash"


module Asteroid
  class Server

    attr_accessor :name,
                  :scripts, 
                  :configure_steps,
                  :instance_options,
                  :ssh_key

    def self.all
      Dir[Asteroid::Config.server_dir + '/**/*.yml'].delete_if do |g|
        File.basename(g) == "default.yml"
      end.map do |g|
        load g
      end
    end

    def self.default_config
      data = YAML.load_file File.join(Asteroid::Config.server_dir, '/default.yml') || {}
      ActiveSupport::HashWithIndifferentAccess.new data
    end

    def self.named(name)
      load Asteroid::Config.server_dir + "/#{name}.yml"
    end

    def self.load(filename)
      data = YAML.load_file filename
      new data
    end

    def initialize(options = {})
      data = self.class.default_config
      data.deep_merge! options

      # Merge environment data
      if env = data.delete(:environments)
        env = env[Asteroid.environment] || {}
        data.deep_merge! env
      end

      @name = data.delete(:name)

      @provider = if data[:provider]
        Provider.named data[:provider]
      else
        Config.providers.first
      end
      
      @instance_options = data.delete :instance_options || {}

      @commands = data.delete(:commands) || {}

      @ssh_key = KeyReference.new data.delete(:ssh_key)

      @config = data["config"] || {
        "admin_username" => "ubuntu",
        "ssh_port" => 1983
      }

      @default_config = data["default_instance_config"] || {
        "login.username" => "root", 
        "ssh.port" => "22"
      }

    end

    def name
      @name && @name.to_sym
    end

    def commands
      @commands
    end

    def [](k)
      config[k.to_s]
    end

    def config
      @config
    end

    def type
      name.to_sym
    end

    def first_instance
      Asteroid::Instance.all_with_type(self.type).first
    end

    def instance_config
      @default_config
    end

    def ssh_key_filename
      if @ssh_key
        File.join Asteroid::Config.secret_key_dir, @ssh_key
      else
        nil
      end
    end

    def provider
      @provider
    end

    def generate_instance_name
      namespace = Config.namespace ? "#{Config.namespace}:" : ""
      "#{namespace}#{self.name}-#{SecureRandom.hex(4)}"
    end

    def create_instance!
      provider.create_instance(self)
    end

  end
end